﻿#Region "FileHeader"
#If False Then
MainModelBase stellt eine singleton-artige shared "Instance"-Property bereit, und einen boolean Instanzen-Member "IsProvisional", welcher angibt, ob das Mainmodel nur provisorisch ist.
Zur Laufzeit soll Instance dasjenige Mainmodel-Unikat im Code global verfügbar machen, welches in App.Xaml zu instanzieren ist (um fürs Xaml global verfügbar zu sein) - dasselbe Mainmodel!

Funktionsweise:
Der Konstruktor setzt den "Singleton".
Er merkt dabei anhand _LazyInitialized, ob er direkt aufgerufen wird oder durch "Singleton"-Abruf. 
Bei letzterem markiert er sich als IsProvisional, und lässt so die Option offen, nochmal direkt aufgerufen zu werden, und den "Singleton" zu überschreiben.
Mehr als ein Direkt-Aufruf (zur Laufzeit) wird aber mit Exception quittiert, denn das Mainmodel muss ein globales Unikat sein.
IsProvisional ist auch im Konstruktor der Erben verfügbar, und zu nutzen, um (möglichst performant) provisorische Test-Daten zu generieren, die dann zur Designzeit im Xaml-Editor sichtbar sind.

Also zur Laufzeit im Code liefert Instance immer ein gültiges Mainmodel. Evtl. ist es aber nur provisorisch, nämlich wenn Instance abgerufen wird, bevor App.Xaml das endgültige Mainmodel erstellt hat.
Somit kann zur Laufzeit maximal ein provisorisches und ein endgültiges Mainmodel erstellt werden - alles andere ist unterbunden.
Zur Designzeit können im Xaml beliebig viele Mainmodels erstellt werden - alle provisorisch.
Man sollte es aber lassen, denn wie gesagt zur Laufzeit knallts dann.
#End If '-- Options, Imports
#End Region 'FileHeader

Namespace System.ComponentModel

   ''' <summary>
   ''' Usage: derive any Mainmodel from MainModelBase(Of T) - eg: Public Class Mainmodel : Inherits MainModelBase(Of Mainmodel)
   ''' </summary>
   <Obsolete("better inherit from NotifyPropertyChanged and instanciate MainModel by Static-Binding (see Template-Solution)")>
   Public Class MainModelBase(Of T As {MainModelBase(Of T), New}) : Inherits NotifyPropertyChanged

      Private Shared _LazyInitialized As Boolean = False
      Private Shared _Instance As T
      Public Shared ReadOnly Property Instance As T
         Get
            If _Instance Is Nothing Then
               _LazyInitialized = True
               _Instance = New T
               _LazyInitialized = False
            End If
            Return _Instance
         End Get
      End Property

      Public ReadOnly IsProvisional As Boolean 'indicates _InstanciatedByInstanceBeforeXaml OrElse IsInDesignMode

      'Private Shadows IsInDesignMode As Boolean = True ' Mock for runtime-tests, which pretends Designmode=True

      Public Sub New()
         IsProvisional = _LazyInitialized OrElse IsInDesignMode
         If IsProvisional OrElse _Instance Is Nothing OrElse _Instance.IsProvisional Then _Instance = DirectCast(Me, T) : Return ' success-exit
         Dim msg = New List(Of Object)
         msg.Add("Only in Designmode you can instantiate several " & GetType(T).Name)
         'msg.Add("IsProvisional: " & IsProvisional)
         'msg.Add("Instance.IsNull: " & _Instance.Null)
         'If _Instance IsNot Nothing Then msg.Add("_Instance.IsProvisional: " & _Instance.IsProvisional)
         'msg.Add("Designmode: " & IsInDesignMode)
         'msg.Add("Process: " & Process.GetCurrentProcess().ProcessName)
         Throw New Exception(String.Join(Lf, msg))
      End Sub

   End Class

End Namespace