﻿#Region "FileHeader"
#If False Then
Viewmodel eines Menus. Man hat vielfältige Möglichkeiten hieraus und aus TreeCommad die Menu-Baumstruktur im Viewmodel zu erstellen, inkl. Commands, Header, CommandParameter, Tooltip, Icon und IsChecked.
Im Xaml braucht man nur 
    <Menu ItemsSource="{Binding Commands}"  Style="{StaticResource commandTreeMenu}"/>
zu binden, und das Menu ist eingerichtet und schickt seine RelayCommands.
Weiters kann man auch an einzelne Commands im CommandTree binden, entweder an MenuItem oder auch an Buttons, unter Angabe der TreeCommand.ID:
      <Button Command="{Binding Commands.01}" />
Dieses richtet Header und Command des Buttons aufs 1.TreeCommand, 2.UnterCommand
Die Einrichtung erfolgt auch wesentlich durch Styles in WpfHelpers\Resources\GeneralStyles.xaml
#End If '-- Options, Imports
Imports System.ComponentModel
#End Region 'FileHeader

Namespace System.Windows.Input
   Public Class CommandTree
      Inherits CustomTypeDescriptor
      Implements IEnumerable(Of TreeCommand)

      Private ReadOnly Root As TreeCommand

      Private _IconSize As Double = 16
      Public Property IconSize() As Double
         Get
            Return _IconSize
         End Get
         Set(value As Double)
            If _IconSize = value Then Return
            _IconSize = value
            Dim recurse As Action(Of TreeCommand) = Sub(tc)
                                                       tc.IconSize = _IconSize
                                                       tc.Children.ForEach(recurse)
                                                    End Sub
            recurse(Root)
         End Set
      End Property


      Private Enum CharIndexMapping
         _0 = 48
         _10 = 87
         _MaxIndex = 65438
      End Enum

      Public Sub New(ParamArray executeMethods As Action())
         Root = TreeCommand.Create("Nop", Nops.Nop)
         Root.IconSize = 16
         Root.ID = ""
         Root.AddRangeFlow(executeMethods)
      End Sub

      Default Public ReadOnly Property Item(hierachicalID As String) As TreeCommand
         Get
            Dim nd = Root
            For Each c In hierachicalID
               nd = nd.Children(GetNodeIndexFromIdChar(c))
            Next
            Return nd
         End Get
      End Property

      ''' <summary> associate ints to chars, assuming a "charset" of: 0123456789abcdefghi... upto char.MaxValue. The resulting MaxIndex is 65438 </summary>
      Private Shared Function GetIdCharFromNodeIndex(index As Integer) As Char
         If index < 0 Then Throw New IndexOutOfRangeException("index must be >= 0")
         If index < 10 Then Return Convert.ToChar(CharIndexMapping._0 + index)
         If index <= CInt(CharIndexMapping._MaxIndex) Then Return Convert.ToChar(CharIndexMapping._10 + index)
         Throw New IndexOutOfRangeException("index must be <= " & CharIndexMapping._MaxIndex)
      End Function

      ''' <summary> associate ints to chars, assuming a "charset" of: 0123456789abcdefghi... upto char.MaxValue. The resulting MaxIndex is 65438 </summary>
      Private Shared Function GetNodeIndexFromIdChar(c As Char) As Integer
         Dim i = Convert.ToInt32(c)
         If i < CharIndexMapping._0 Then Throw New IndexOutOfRangeException("index must be >= '0'")
         If i < CharIndexMapping._10 Then Return i - CharIndexMapping._0
         If i < CharIndexMapping._MaxIndex Then Return i - CharIndexMapping._10
         Throw New IndexOutOfRangeException("index must be <= Convert.ToChar(" & CharIndexMapping._MaxIndex & ")")
      End Function

      ''' <summary> set a unique ID to each TreeCommand, return all Commands as flat IEnumerable. The ID is hierarchical and human readable. Eg "021" addresses the first node, and from that the third child, and from that the second child </summary>
      Public Function SetupIDs() As IEnumerable(Of TreeCommand)
         'breath-first-traversal with assigning new IDs
         Dim lst = New List(Of TreeCommand)()
         Dim addRange As Action(Of TreeCommand) = Sub(parent)
                                                     Dim childs = parent.Children
                                                     Dim id = parent.ID
                                                     For i = 0 To childs.Count - 1
                                                        childs(i).ID = id + GetIdCharFromNodeIndex(i)
                                                     Next
                                                     lst.AddRange(childs)
                                                  End Sub
         addRange(Root)
         For i = 0 To lst.Count - 1
            addRange(lst(i))
         Next
         Return lst.Distinct()
      End Function

      ''' <summary> for internal use, to support Databinding </summary>
      <EditorBrowsable(EditorBrowsableState.Never)> _
      Public Overrides Function GetProperties() As PropertyDescriptorCollection
         'unconventional, but efficient: the PropertyDescriptor is a Member of the Property-Datatype
         Return New PropertyDescriptorCollection(SetupIDs().Select(Function(cmd) cmd.Descriptor).ToArray())
      End Function
      ''' <summary> for internal use, to support Databinding </summary>
      <EditorBrowsable(EditorBrowsableState.Never)> _
      Public Overrides Function GetProperties(attributes As Attribute()) As PropertyDescriptorCollection
         Return GetProperties()
      End Function

      Private Function IEnumerable_GetEnumerator() As IEnumerator(Of TreeCommand) Implements IEnumerable(Of TreeCommand).GetEnumerator
         Return Root.Children.GetEnumerator()
      End Function

      Private Function Collections_IEnumerable_GetEnumerator() As Collections.IEnumerator Implements Collections.IEnumerable.GetEnumerator
         Return Root.Children.GetEnumerator()
      End Function
   End Class
End Namespace
