﻿Imports System.ComponentModel
Imports System.Globalization

Namespace System.Windows.Input
   Public Class RelayCommand : Inherits CommandBase
      Private ReadOnly _execute As Action
      Private ReadOnly _canExecute As Func(Of Boolean)
      <DebuggerStepThrough()> _
      Public Sub New(execute As Action, Optional canExecute As Func(Of Boolean) = Nothing)
         If execute Is Nothing Then Throw New ArgumentNullException("execute")
         Me._execute = execute
         _canExecute = If(canExecute, Function() Enabled)
      End Sub
      <DebuggerStepThrough()> _
      Public Overrides Function CanExecute(parameter As Object) As Boolean
         Return _canExecute()
      End Function
      <DebuggerStepThrough()> _
      Public Overrides Sub Execute(parameter As Object)
         Me._execute()
      End Sub
   End Class

   Public Class RelayCommand(Of T) : Inherits CommandBase
      Private ReadOnly _execute As Action(Of T)
      Private ReadOnly _canExecute As Func(Of T, Boolean)
      Private ReadOnly _Conv As TypeConverter = TypeDescriptor.GetConverter(GetType(T))
      <DebuggerStepThrough()> _
      Public Sub New(execute As Action(Of T), Optional canExecute As Func(Of T, Boolean) = Nothing)
         If execute Is Nothing Then Throw New ArgumentNullException("execute")
         Me._execute = execute
         _canExecute = canExecute
      End Sub
      Private Function Convert(p As Object) As T
         If TypeOf p Is T Then Return DirectCast(p, T)
         If p Is Nothing Then Return Nothing
         Return DirectCast(_Conv.ConvertFrom(Nothing, CultureInfo.InvariantCulture, p), T)
      End Function
      <DebuggerStepThrough()> _
      Public Overrides Function CanExecute(parameter As Object) As Boolean
         Return If(_canExecute.Null, Enabled, _canExecute(Convert(parameter)))
      End Function
      <DebuggerStepThrough()> _
      Public Overrides Sub Execute(parameter As Object)
         Me._execute(Convert(parameter))
      End Sub
   End Class

   Public MustInherit Class CommandBase : Inherits NotifyPropertyChanged : Implements ICommand

      Private EventDisabled As New Counter 'verhindert Ereigniskette beim gegenseitigen Aufruf von CanExecute und Setter der Enabled-Property

      'das CanExecuteChanged-Event vereinigt sich mit dem statischen CommandManager.RequerySuggested-Event. Eiglich müsste es LaunchRequery heißen oderso 
      Public Custom Event CanExecuteChanged As EventHandler Implements ICommand.CanExecuteChanged
         AddHandler(value As EventHandler)
            AddHandler CommandManager.RequerySuggested, value
         End AddHandler
         RemoveHandler(value As EventHandler)
            RemoveHandler CommandManager.RequerySuggested, value
         End RemoveHandler
         RaiseEvent(sender As Object, e As EventArgs)
            CommandManager.InvalidateRequerySuggested()
         End RaiseEvent
      End Event

      Public Shared ReadOnly EnabledChangedArgs As New PropertyChangedEventArgs("Enabled")

      Public MustOverride Sub Execute(parameter As Object) Implements ICommand.Execute
      Public MustOverride Function CanExecute(parameter As Object) As Boolean
      Private _Enabled As Boolean = True
      Public Property Enabled() As Boolean
         Get
            Return _Enabled
         End Get
         Set(value As Boolean)
            If ChangePropIfDifferent(value, _Enabled, EnabledChangedArgs) Then
               If EventDisabled.Up = 0 Then RaiseEvent CanExecuteChanged(Me, EventArgs.Empty)
               EventDisabled.Down()
            End If
         End Set
      End Property
      Private Function ICanExecute(parameter As Object) As Boolean Implements ICommand.CanExecute
         If EventDisabled.Up = 0 Then Enabled = CanExecute(parameter)
         EventDisabled.Down()
         Return _Enabled
      End Function
   End Class
End Namespace
