﻿#Region "FileHeader"
#If False Then
Dieses ist Viewmodel für ein MenuItem - s.a. CommandTree-FileHeader 
#End If '-- Options, Imports
Imports System.ComponentModel
Imports System.Windows.Media
#End Region 'FileHeader

Namespace System.Windows.Input
   <DebuggerDisplay("{ID}, {Header}")> _
   Public Class TreeCommand : Inherits NotifyPropertyChanged : Implements ICommand, IEnumerable(Of TreeCommand)

      Public Event CheckedChanged As EventHandler

      ''' <summary> The ID is hierarchical and human readable. Eg "021" addresses the first node, and from that the third child, and from that the second child </summary>
      Public Property ID() As String = "ID-Placeholder"

      Public Property Header() As Object
         Get
            Return _Header
         End Get
         Set(value As Object)
            ChangePropIfDifferent(value, _Header)
         End Set
      End Property
      Private _Header As Object

      Public ReadOnly Property Icon() As Object ' dieses kann nicht gecached werden - daher beim binden auf ,Mode=OneTime achten
         Get
            If IconSource.Null Then Return Nothing
            Try
               Return New Image With {.Source = New Windows.Media.Imaging.BitmapImage(New Uri(IconSource)), .Width = IconSize}
            Catch ex As Exception
               Return New Image With {.Source = ErrorImage.Instance, .Width = IconSize}
            End Try
         End Get
      End Property

      Public Property ToolTip() As Object
         Get
            Return _ToolTip
         End Get
         Set(value As Object)
            ChangePropIfDifferent(value, _ToolTip)
         End Set
      End Property
      Private _ToolTip As Object

      Private _IsCheckable As Boolean
      Public Property IsCheckable() As Boolean
         Get
            Return _IsCheckable
         End Get
         Set(value As Boolean)
            ChangePropIfDifferent(value, _IsCheckable)
         End Set
      End Property

      Private _IsChecked As Boolean
      Public Property IsChecked() As Boolean
         Get
            Return _IsChecked
         End Get
         Set(value As Boolean)
            If ChangePropIfDifferent(value, _IsChecked) Then RaiseEvent CheckedChanged(Me, EventArgs.Empty)
         End Set
      End Property

      Public Function SetChecked(value As Boolean?) As TreeCommand
         IsCheckable = value.HasValue
         If _IsCheckable Then IsChecked = value.Value
         Return Me
      End Function

      Private _Command As CommandBase
      Public ReadOnly Descriptor As DelegatePropertyDescriptor

      Friend IconSize As Double
      Public IconSource As String = Nothing


      Public ReadOnly Property Children() As List(Of TreeCommand)
         Get
            Return _Children
         End Get
      End Property
      Private _Children As New List(Of TreeCommand)()

      Private Function IEnumerable_GetEnumerator() As IEnumerator(Of TreeCommand) Implements IEnumerable(Of TreeCommand).GetEnumerator
         Return Children.GetEnumerator()
      End Function
      Private Function Collections_IEnumerable_GetEnumerator() As Collections.IEnumerator Implements Collections.IEnumerable.GetEnumerator
         Return Children.GetEnumerator()
      End Function

      ''' <summary> returns the added TreeCommand </summary>
      Public Function Add(chld As TreeCommand) As TreeCommand
         chld.IconSize = IconSize
         Children.Add(chld)
         Return chld
      End Function
      ''' <summary> create, add and return a TreeCommand </summary>
      Public Function Add(executeMethod As Action, Optional canExecuteMethod As Func(Of Boolean) = Nothing) As TreeCommand
         Return Add(TreeCommand.Create(executeMethod, canExecuteMethod))
      End Function
      ''' <summary> create, add and return a TreeCommand </summary>
      Public Function Add(executeMethod As Action(Of Object), Optional canExecuteMethod As Func(Of Object, Boolean) = Nothing) As TreeCommand
         Return Add(TreeCommand.Create(executeMethod, canExecuteMethod))
      End Function
      ''' <summary> create, add and return a TreeCommand </summary>
      Public Function Add(commandName As String, Optional canExecuteMethod As Func(Of Boolean) = Nothing) As TreeCommand
         Return Add(TreeCommand.Create(commandName, AddressOf Nop, canExecuteMethod))
      End Function
      ''' <summary> create, add and return a TreeCommand </summary>
      Public Function Add(commandName As String, executeMethod As Action, Optional canExecuteMethod As Func(Of Boolean) = Nothing) As TreeCommand
         Return Add(TreeCommand.Create(commandName, executeMethod, canExecuteMethod))
      End Function
      ''' <summary> create, add and return a TreeCommand </summary>
      Public Function Add(commandName As String, executeMethod As Action(Of Object), Optional canExecuteMethod As Func(Of Object, Boolean) = Nothing) As TreeCommand
         Return Add(TreeCommand.Create(commandName, executeMethod, canExecuteMethod))
      End Function
      ''' <summary> returns the TreeCommand at the index </summary>
      Default Public ReadOnly Property Item(index As Integer) As TreeCommand
         Get
            Return Children(index)
         End Get
      End Property

      ''' <summary> create and add several TreeCommands, returns the Parent of them </summary>
      Public Function AddRangeFlow(ParamArray executeMethods As Action()) As TreeCommand
         For Each act In executeMethods : Add(act) : Next : Return Me
      End Function

      ''' <summary> create and add several TreeCommands, returns the Parent of them </summary>
      Public Function AddRangeFlow(ParamArray executeMethods As Action(Of Object)()) As TreeCommand
         For Each act In executeMethods : Add(act) : Next : Return Me
      End Function

      ''' <summary> Flow-Interface: create and add a child-TreeCommand, then the parent returns itself </summary>
      Public Function AddFlow(executeMethod As Action(Of Object), Optional canExecuteMethod As Func(Of Object, Boolean) = Nothing) As TreeCommand
         Return AddFlow(executeMethod.Method.Name, executeMethod, canExecuteMethod)
      End Function
      ''' <summary> Flow-Interface: create and add a child-TreeCommand, then the parent returns itself </summary>
      Public Function AddFlow(commandName As String, executeMethod As Action(Of Object), Optional canExecuteMethod As Func(Of Object, Boolean) = Nothing) As TreeCommand
         Return AddFlow(TreeCommand.Create(commandName, executeMethod, canExecuteMethod))
      End Function

      ''' <summary> Flow-Interface: create and add a child-TreeCommand, then the parent returns itself </summary>
      Public Function AddFlow(commandName As String) As TreeCommand
         Return AddFlow(commandName, Nops.Nop, Nothing)
      End Function
      ''' <summary> Flow-Interface: create and add a child-TreeCommand, then the parent returns itself </summary>
      Public Function AddFlow(executeMethod As Action, Optional canExecuteMethod As Func(Of Boolean) = Nothing) As TreeCommand
         Return AddFlow(executeMethod.Method.Name, executeMethod, canExecuteMethod)
      End Function
      ''' <summary> Flow-Interface: create and add a child-TreeCommand, then the parent returns itself </summary>
      Public Function AddFlow(commandName As String, executeMethod As Action, Optional canExecuteMethod As Func(Of Boolean) = Nothing) As TreeCommand
         Return AddFlow(TreeCommand.Create(commandName, executeMethod, canExecuteMethod))
      End Function

      ''' <summary> Flow-Interface: add the child-TreeCommand, then the parent returns itself </summary>
      Public Function AddFlow(child As TreeCommand) As TreeCommand
         Add(child)
         Return Me
      End Function

      ''' <summary> Indexer-Alienation as Flow-Interface: create and add a TreeCommand, then the addressed parent returns itself </summary>
      Default Public ReadOnly Property Item(commandName As String) As TreeCommand
         Get
            Return AddFlow(commandName)
         End Get
      End Property
      ''' <summary> Indexer-Alienation as Flow-Interface: create and add a TreeCommand, then the addressed parent returns itself </summary>
      Default Public ReadOnly Property Item(executeMethod As Action, Optional canExecuteMethod As Func(Of Boolean) = Nothing) As TreeCommand
         Get
            Return AddFlow(executeMethod, canExecuteMethod)
         End Get
      End Property
      ''' <summary> Indexer-Alienation as Flow-Interface: create and add a TreeCommand, then the addressed parent returns itself </summary>
      Default Public ReadOnly Property Item(commandName As String, executeMethod As Action, Optional canExecuteMethod As Func(Of Boolean) = Nothing) As TreeCommand
         Get
            Return AddFlow(commandName, executeMethod, canExecuteMethod)
         End Get
      End Property
      ''' <summary> Indexer-Alienation as Flow-Interface: add the child-TreeCommand, then the addressed parent returns itself </summary>
      Default Public ReadOnly Property Item(child As TreeCommand) As TreeCommand
         Get
            Return AddFlow(child)
         End Get
      End Property

      Private Sub New()
         Descriptor = New DelegatePropertyDescriptor(Function() ID, Function() Me)
      End Sub

      Public Shared Function Create(executeMethod As Action, Optional canExecuteMethod As Func(Of Boolean) = Nothing) As TreeCommand
         Return Create(executeMethod.Method.Name, executeMethod, canExecuteMethod)
      End Function
      Public Shared Function Create(header As Object, executeMethod As Action, Optional canExecuteMethod As Func(Of Boolean) = Nothing) As TreeCommand
         Dim tcmd = New TreeCommand() With {._Command = New RelayCommand(executeMethod, canExecuteMethod)}
         Dim commandName = TryCast(header, String)
         If commandName IsNot Nothing Then
            tcmd.Header = commandName
         Else
            tcmd.Header = header
         End If
         Return tcmd
      End Function

#If True Then
      Public Shared Function Create(executeMethod As Action(Of Object), Optional canExecuteMethod As Func(Of Object, Boolean) = Nothing) As TreeCommand
         Return Create(executeMethod.Method.Name, executeMethod, canExecuteMethod)
      End Function
      Public Shared Function Create(header As Object, executeMethod As Action(Of Object), Optional canExecuteMethod As Func(Of Object, Boolean) = Nothing) As TreeCommand
         Dim tcmd = New TreeCommand() With {._Command = New RelayCommand(Of Object)(executeMethod, canExecuteMethod)}
         Dim commandName = TryCast(header, String)
         If commandName IsNot Nothing Then
            tcmd.Header = commandName
         Else
            tcmd.Header = header
         End If
         Return tcmd
      End Function
#ElseIf True Then
#ElseIf True Then
#End If


      Public Custom Event CanExecuteChanged As EventHandler Implements ICommand.CanExecuteChanged
         AddHandler(value As EventHandler)
            AddHandler _Command.CanExecuteChanged, value
         End AddHandler
         RemoveHandler(value As EventHandler)
            RemoveHandler _Command.CanExecuteChanged, value
         End RemoveHandler
         RaiseEvent(sender As Object, e As EventArgs)
            CommandManager.InvalidateRequerySuggested()
         End RaiseEvent
      End Event

      Public Function CanExecute(parameter As Object) As Boolean Implements ICommand.CanExecute
         Return _Command.CanExecute(If(parameter, Me))
      End Function

      Public Sub Execute(parameter As Object) Implements ICommand.Execute
         _Command.Execute(If(parameter, Me))
      End Sub

      Public Shared ReadOnly EnabledChangedArgs As New PropertyChangedEventArgs("Enabled")
      Public Property Enabled() As Boolean
         Get
            Return _Command.Enabled
         End Get
         Set(value As Boolean)
            Dim enbl = _Command.Enabled
            If ChangePropIfDifferent(value, enbl, EnabledChangedArgs) Then _Command.Enabled = enbl
         End Set
      End Property
   End Class

End Namespace
