﻿#Region "FileHeader"
#If False Then
2 (Basis-)Klassen: Node(Of T) + Tree(Of T), für den Aufbau baumartiger Datenstrukturen.
Funktionalitäts-Bereiche:
Node erbt Resetlist und fügt Name und Parent hinzu. 
DataBindablität mit ChangePropIfDifferent()
SetItem, Insert, AddRange, ClearItems - Overrides sorgen für Daten-Konsistenz, also dass der Parent immer richtig gesetzt ist, und ein Node nicht in mehreren anneren Node enthalten sein kann (denn welchen Parent sollte er dann haben?).
Es gibt 3 NodeTypes: TopLevel, Inner, Leaf. Root ist kein NodeTyp, denn der Root-Node tritt niemals an die Oberfläche.
Methoden Anchestors() und GetLeafs() rufen die Anchestors (inklusive Root) bzw. enumerieren die Leafs.
Die Tree(Of T)-Instanz ist per WeakReference nur an den Root attachet. Dort sind Member, die für alle Nodes gleichermaßen relevant sind, zunächstmal der SelectedNode. Tree(Of T)-Erben können dort weiteres hinterlegen.
Natürlich können Node-Erben auch weitere Node-Spezifika anlegen.
Tree.FindNode(path) sucht im ganzen Tree, und die Überladung mit pathSegments liefert bei Nichtfinden immerhin den Node, wo der Rest-Pfad hineingehören würde. pathSegments As IEnumerator(Of T) ist aber bisserl unkonventionell zu benutzen.
Beachte auch die Klasse SortedNode(Of T), die Node(Of T) beerbt und das Verhalten modifiziert, sodass der Tree nach Name sortiert ist.
Für riesige DatenMengen mit hunderttausenden von Leafs wäre die Architektur optimierbar, indem man statt der fetten BasisBasisKlasse ObservableCollection auf Composition setzt, und mittels WeakReferenz nur den Nicht-Leaf-Nodes eine Child-Property attacht, sodass die Leafs keinerlei Speicher verbraten für Childs, die sie nicht haben.
#End If '-- Options, Imports
Imports System.ComponentModel
Imports System.Runtime.CompilerServices
#End Region 'FileHeader

Namespace System

   Public Enum NodeType : Inner : TopLevel : Leaf : TopLevelLeaf : End Enum

   <DebuggerDisplay("{FullName}")> _
   Public Class Node(Of TNode As {Node(Of TNode), New}) : Inherits ResetList(Of TNode)

      Protected _Name As String = ""
      Public Overridable Property Name As String
         Get
            Return _Name
         End Get
         Set(value As String)
            ChangePropIfDifferent(value, _Name, "Name", "FullName")
         End Set
      End Property
      Public Property Parent As Node(Of TNode)

      Private Shared _Ancestors As New Stack(Of Node(Of TNode))(20)
      Public Function Ancestors() As Stack(Of Node(Of TNode))
         _Ancestors.Clear()
         Dim p = Parent
         While p.NotNull
            _Ancestors.Push(p)
            p = p.Parent
         End While
         Return _Ancestors
      End Function
      Public ReadOnly Property FullName() As String
         Get
            Return String.Concat(Ancestors.Skip(1).Select(Function(nd) nd.Name & "\")) & Name
         End Get
      End Property

      Public Overrides Sub AddRange(news As IEnumerable(Of TNode), Optional startIndex As Integer = 2147483647)
         ' Da base.AddRange sich an InsertItem vorbeimogelt, musses überschrieben werden, um Logik-Konsistenz zu gewähren
         MyBase.AddRange(news, startIndex)
         For Each nd In news : nd.Parent = Me : Next 'trägt sich selbst als nd.Parent ein
         With TryCast(news, Node(Of TNode)) ' noch weitere Logik: Falls news ein TNode ist, werden dessen Childs gelöscht
            If .NotNull Then .InnerList.Clear()
         End With
      End Sub

      Protected Overrides Sub InsertItem(index As Integer, nd As TNode)
         'trägt sich selbst als nd.Parent ein
         If nd.Parent.NotNull Then nd.Parent.Remove(nd)
         nd.Parent = Me
         MyBase.InsertItem(index, nd)
         If Count = 1 Then OnPropertyChanged(New PropertyChangedEventArgs("Type"))
      End Sub
      Protected Overrides Sub RemoveItem(index As Integer)
         Dim itm = Me(index)
         MyBase.RemoveItem(index)
         If Count = 0 Then OnPropertyChanged(New PropertyChangedEventArgs("Type"))
         itm.Parent = Nothing
      End Sub
      Protected Overrides Sub ClearItems()
         If Count = 0 Then Return
         For Each itm In Me
            itm._Parent = Nothing
         Next
         MyBase.ClearItems()
         OnPropertyChanged(New PropertyChangedEventArgs("Type"))
      End Sub
      Protected Overrides Sub SetItem(index As Integer, item As TNode)
         Dim itm = Me(index)
         MyBase.SetItem(index, item)
         item.Parent = Me
         itm.Parent = Nothing
      End Sub
      Public ReadOnly Property Type As NodeType
         Get
            Dim tp = If(Parent.Parent.Null, NodeType.TopLevel, NodeType.Inner)
            Return If(Count > 0, tp, tp Or NodeType.Leaf)
         End Get
      End Property

      Default Public Overloads ReadOnly Property Item(name As String) As TNode
         Get
            Dim i = FindIndexByName(name)
            If i < 0 Then Throw New KeyNotFoundException("'".Between(Additional.Start, name, " in ", FullName, " nicht gefunden!"))
            Return Me(i)
         End Get
      End Property

      Public Overridable Function FindIndexByName(name As String) As Integer
         Return InnerList.FindIndex(Function(nd) nd.Name = name)
      End Function

      Public Overrides Function ToString() As String
         Return FullName
      End Function

      Public Function GetLeaves() As List(Of TNode)
         Dim lst = New List(Of TNode)
         Dim recurse As Action(Of TNode) = _
            Sub(nd)
               If nd.Count > 0 Then
                  nd.ForEach(recurse)
               Else
                  lst.Add(nd)
               End If
            End Sub
         recurse(DirectCast(Me, TNode))
         Return lst
      End Function

      Public Property IsSelected() As Boolean
         Get
            Return Tree.SelectedNode Is Me
         End Get
         Set( value As Boolean)
            Dim tree = Me.Tree
            If value = (tree.SelectedNode Is Me) Then Return
            tree.SelectedNode = If(value, DirectCast(Me, TNode), Nothing)
            RaisePropChanged("IsSelected")
         End Set
      End Property
      Public Overridable Function InsertNewNodePath(index As Integer, segments As IEnumerator(Of String)) As TNode
         Dim ndFirst = New TNode With {.Name = segments.Current}
         InsertNewNodePath = ndFirst
         While segments.MoveNext
            Dim chlds = InsertNewNodePath
            InsertNewNodePath = New TNode With {.Name = segments.Current, .Parent = InsertNewNodePath}
            chlds.Insert(0, InsertNewNodePath)
         End While
         Insert(index, ndFirst)
      End Function
      Public Sub Delete()
         Parent.Remove(DirectCast(Me, TNode))
      End Sub

      Public Overridable Sub DeleteBranch()
         Dim ancs = Ancestors(), i As Integer
         For i = ancs.Count - 1 To 1 Step -1 'vom Parent bis zum TopLevel, sodass i nach der Schleife minimal auf 0 - Root stehen kann
            If ancs(i).Count > 1 Then Exit For
         Next
         Call If(i = ancs.Count - 1, Me, ancs(i + 1)).Delete() 'wurde garnet decrementiert, mussich selbst gelöscht wern.
      End Sub

      Public Overridable Sub DeleteSaveChilds()
         If Count = 0 Then Delete() : Return
         Dim i = Parent.IndexOf(DirectCast(Me, TNode))
         If Count = 1 Then Parent(i) = Me(0) : Return
         Dim parentNodes = Parent.InnerList
         parentNodes.RemoveAt(i)
         parentNodes.InsertRange(i, Me)
         Parent.RaiseReset()
      End Sub
      Private Shared _Tree As New ConditionalWeakTable(Of Node(Of TNode), Tree(Of TNode))
      Public Property Tree() As Tree(Of TNode)
         Get
            Dim tr As Tree(Of TNode) = Nothing
            Dim nd = Ancestors(0)
            If _Tree.TryGetValue(nd, tr) Then Return tr
            Return Nothing
         End Get
         Friend Set( value As Tree(Of TNode))
            If Parent.NotNull Then Throw Me.Exception("only set Tree to Root-Nodes!")
            Dim nd = DirectCast(Me, TNode)
            _Tree.Remove(nd)
            _Tree.Add(nd, value)
         End Set
      End Property

   End Class

End Namespace