﻿Imports System.Collections.Generic
Imports System.Windows.Media
Imports System.Collections
Imports System.Diagnostics
Imports System.Reflection
Imports System.Runtime.CompilerServices

Namespace System.Windows.Controls
   Public Module DependencyObjectX

      <Extension()> _
      Public Function FindChild(Of T As DependencyObject)(parent As DependencyObject) As T
         For i = 0 To VisualTreeHelper.GetChildrenCount(parent) - 1
            Dim child = VisualTreeHelper.GetChild(parent, i)
            Dim typedChild = If(TryCast(child, T), child.FindChild(Of T)())
            If typedChild IsNot Nothing Then Return typedChild
         Next
         Return Nothing
      End Function
      <Extension()> _
      Public Function FindLogChild(Of T As DependencyObject)(parent As DependencyObject) As T
         Dim queue = New Queue(Of DependencyObject)
         queue.Enqueue(parent)
         While queue.Count > 0
            For Each d As DependencyObject In LogicalTreeHelper.GetChildren(queue.Dequeue)
               FindLogChild = d.TryCastX(Of T)()
               If FindLogChild.NotNull Then Exit Function
               queue.Enqueue(d)
            Next
         End While
         Return Nothing
      End Function
      
      ''' <summary> bubble up from e.OriginalSource to e.Source and search a matching DataContext </summary>
      <Extension()> _
      Public Function FindParentContext(Of TViewmodel As Class)(e As RoutedEventArgs) As TViewmodel
         Dim result As TViewmodel = Nothing
         Dim parent = DirectCast(e.OriginalSource, DependencyObject)
         Do
            Dim fwe = TryCast(parent, FrameworkElement)
            If fwe.Null Then
               Dim fwce = TryCast(parent, FrameworkContentElement)
               If fwce.NotNull AndAlso result.TryBe(fwce.DataContext) Then Return result
            Else
               If result.TryBe(fwe.DataContext) Then Return result
            End If
            parent = VisualTreeHelper.GetParent(parent)
         Loop Until parent Is Nothing OrElse parent Is e.Source
         Return Nothing
      End Function

      <Extension()> _
      Public Function  FindParent(Of T As DependencyObject)(obj As DependencyObject, Optional root As DependencyObject = Nothing) As T
         Do
            Try
               obj = VisualTreeHelper.GetParent(obj)
            Catch x As InvalidOperationException
               Const IsNoVisualException As Integer = -2146233079               ' not 2010-compatible
               If x.HResult <> IsNoVisualException Then Throw ' not 2010-compatible
               obj = LogicalTreeHelper.GetParent(obj)
            End Try
            If obj Is Nothing OrElse obj Is root Then Return Nothing
            Dim result = TryCast(obj, T)
            If result IsNot Nothing Then Return result
         Loop
      End Function

      <Extension()> _
      Public Function FindParent(obj As DependencyObject, tp As Type) As DependencyObject
         Do
            obj = VisualTreeHelper.GetParent(obj)
            If obj.Null Then Return Nothing
            If obj.GetType Is tp Then Return obj
         Loop
      End Function

      Private _DefaultSelector As DataTemplateSelector = Nothing

      <Extension()> _
      Public Function GetDefaultDataTemplate(d As DependencyObject, itm As Object) As DataTemplate
         If _DefaultSelector.Null() Then
            Dim tp As Type = GetType(ContentPresenter).Assembly.GetType("ContentPresenter+DefaultSelector")
            _DefaultSelector = DirectCast(tp.GetConstructor(Type.EmptyTypes).Invoke(Nothing), DataTemplateSelector)
         End If
         Return _DefaultSelector.SelectTemplate(itm, d)
      End Function

      <Extension()> _
      Public Sub DumpVisual(parent As DependencyObject)
         Debug.Print("VisualDump of {0}", parent.[GetType]().Name)
         Dim dumpOn As Action(Of DependencyObject) = Nothing
         Debug.IndentSize = 3
         dumpOn = Sub(dpd)
                     Debug.Indent()
                     For i = 0 To VisualTreeHelper.GetChildrenCount(dpd) - 1
                        Dim child = VisualTreeHelper.GetChild(dpd, i)
                        Debug.Print("{0} {1,2}", child.[GetType]().Name, i)
                        dumpOn(child)
                     Next
                     Debug.Unindent()
                  End Sub
         dumpOn(parent)
      End Sub

      <Extension()> _
      Public Sub DumpLogical(parent As DependencyObject)
         Debug.Print("LogicalDump of {0}", parent.[GetType]().Name)
         Dim dumpOn As Action(Of DependencyObject) = Nothing
         Debug.IndentSize = 3
         dumpOn = Sub(dpd)
                     Debug.Indent()
                     For Each child In LogicalTreeHelper.GetChildren(dpd)
                        Debug.Print(child.[GetType]().Name)
                        With TryCast(child, DependencyObject)
                           If .NotNull Then dumpOn(.Self)
                        End With
                     Next
                     Debug.Unindent()
                  End Sub
         dumpOn(parent)
      End Sub

      <Extension()> _
      Public Function GetChild(parent As DependencyObject, indexes As IEnumerable(Of Integer)) As DependencyObject
         For Each i In indexes
            parent = VisualTreeHelper.GetChild(parent, i)
         Next
         Return parent
      End Function
   End Module
End Namespace
